@@ -16,7 +16,7 @@ from stardist.models import StarDist2D
 # import skimage.io
 # from stardist.models import StarDist3D
 from csbdeep.utils import normalize
-from naparimovie import Movie
+#from naparimovie import Movie
 import math
 import gc
 
@@ -52,7 +52,7 @@ class NapariSTPT:
         self.im1 = None
         self.im32 = None
 
-        self.movie = None
+        #self.movie = None
         self.optical_slices = None
         self.nr_optical_slices = None
         self.shape = None
@@ -63,7 +63,7 @@ class NapariSTPT:
         self.crop_size_ratio_y = None
 
         self.crop = False
-
+        
     def CropOriginal(self, volume):
 
         threshold = float(self.thresholdN.text())
@@ -127,17 +127,19 @@ class NapariSTPT:
         # ds4 = xr.open_zarr(str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.4')
         # ds8 = xr.open_zarr(str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.8')
         # ds32 = xr.open_zarr(str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.32')
+        
+        
 
         self.slice_names = self.ds1.attrs['cube_reg']['slice']
         # print(self.slice_names)
 
         bscale = self.ds1.attrs['bscale']
         bzero = self.ds1.attrs['bzero']
-
-        # print(self.ds1.attrs.keys())
-        # print(self.ds1.attrs)
-        # print(self.ds1['S001'].attrs.keys())
-        # print(self.ds1['S001'].attrs)
+        
+        #print(self.ds1['S001'].attrs['raw_meta'][0][0]['zres'])
+        #print(self.ds1.keys())
+        #print(self.ds1.attrs.keys())
+        #print(self.ds1['S001'].attrs.keys())
         # print(self.ds1.attrs["multiscale"]['datasets'][1]['level'])
         # print(self.ds1.attrs["multiscale"]['datasets'][1]['path'])
 
@@ -147,9 +149,28 @@ class NapariSTPT:
 
         # print("size: {}".format(self.ds1.dims))
 
-        resolution = int(self.comboBoxResolution.currentText())
+        output_resolution = float(self.pixel_size.text())
+        #resolution = int(self.comboBoxResolution.currentText())
+        resolution = 32
+        index = 5
+        if (output_resolution / 0.5) < 32:
+            resolution = 16
+            index = 4
+        if (output_resolution / 0.5) < 16:
+            resolution = 8
+            index = 3
+        if (output_resolution / 0.5) < 8:
+            resolution = 4
+            index = 2
+        if (output_resolution / 0.5) < 4:
+            resolution = 2
+            index = 1
+        if (output_resolution / 0.5) < 2:
+            resolution = 1
+            index = 0
+        
         print("loading at resolution {}".format(resolution))
-        index = self.comboBoxResolution.currentIndex()
+        #index = self.comboBoxResolution.currentIndex()
         print("group: " +
               self.ds1.attrs["multiscale"]['datasets'][index]['path'])
         gr = self.ds1.attrs["multiscale"]['datasets'][index]['path']
@@ -157,12 +178,18 @@ class NapariSTPT:
         ) + str(self.comboBoxPath.currentText())+'/mos.zarr', group=gr)
 
         self.optical_slices = len(ds.z)
-        if self.optical_slices > int(self.nr_optical_slices.text()):
-            self.optical_slices = int(self.nr_optical_slices.text())
+        print(f"optical_slices available: {self.optical_slices}")
+        
+        if self.optical_slices > 1:
+            optical_section_spacing = self.ds1['S001'].attrs['raw_meta'][0][0]['zres']
+            print(f"optical_slices zres: {optical_section_spacing}")
+            expected_nr_of_slices = round(15.0 / optical_section_spacing)
+            if self.optical_slices > expected_nr_of_slices:
+                self.optical_slices = expected_nr_of_slices
 
-        print(f"optical_slices: {self.optical_slices}")
+        print(f"optical_slices used: {self.optical_slices}")
 
-        output_resolution = float(self.pixel_size.text())
+        
 
         with napari.gui_qt():
             if self.cb_C1.isChecked():
@@ -331,9 +358,9 @@ class NapariSTPT:
                     15/self.optical_slices, output_resolution, output_resolution), blending='additive', colormap='bop blue', translate=(0, int(start_y*output_resolution), int(start_z*output_resolution)))
 
             # create naparimovie object
-            if self.movie is None:
-                self.movie = Movie(myviewer=self.viewer)
-                self.movie.inter_steps = 30
+            #if self.movie is None:
+                #self.movie = Movie(myviewer=self.viewer)
+                #self.movie.inter_steps = 30
 
     def LoadInRegion(self, text):
 
@@ -380,8 +407,8 @@ class NapariSTPT:
         # print("crop to: {} {} {} {}".format(minX, maxX, minY, maxY))
 
         self.crop = True
-
         self.Load(text)
+        self.crop = False
 
     def Load2D(self, text):
 
@@ -431,13 +458,13 @@ class NapariSTPT:
                 channel=1, type='mosaic', z=0).data * bscale + bzero
             im8 = ds8[slice_name].sel(
                 channel=1, type='mosaic', z=0).data * bscale + bzero
-            im16 = self.ds16[slice_name].sel(
+            im16 = ds16[slice_name].sel(
                 channel=1, type='mosaic', z=0).data * bscale + bzero
-            im32 = ds32[slice_name].sel(
+            self.im32 = ds32[slice_name].sel(
                 channel=1, type='mosaic', z=0).data * bscale + bzero
 
             with napari.gui_qt():
-                self.viewer.add_image([im1, im2, im4, im8, im16, im32], multiscale=True,
+                self.viewer.add_image([im1, im2, im4, im8, im16, self.im32], multiscale=True,
                                       name='C1', blending='additive', colormap='bop purple')
 
         if self.cb_C2.isChecked():
@@ -449,7 +476,7 @@ class NapariSTPT:
                 channel=2, type='mosaic', z=0).data * bscale + bzero
             im8 = ds8[slice_name].sel(
                 channel=2, type='mosaic', z=0).data * bscale + bzero
-            im16 = self.ds16[slice_name].sel(
+            im16 = ds16[slice_name].sel(
                 channel=2, type='mosaic', z=0).data * bscale + bzero
             im32 = ds32[slice_name].sel(
                 channel=2, type='mosaic', z=0).data * bscale + bzero
@@ -475,7 +502,7 @@ class NapariSTPT:
             with napari.gui_qt():
                 self.viewer.add_image([im1, im2, im4, im8, im16, im32], multiscale=True,
                                       name='C3', blending='additive', colormap='green')
-                # self.viewer.add_image(float_img, multiscale=False, name='C3', blending='additive', colormap='green')
+                #self.viewer.add_image(im4, multiscale=False, name='C3', blending='additive', colormap='green')
 
         if self.cb_C4.isChecked():
             im1 = self.ds1[slice_name].sel(
@@ -507,8 +534,8 @@ class NapariSTPT:
         size2D = (
             int(size_multiplier*volume.shape[2]), int(size_multiplier*volume.shape[1]))
 
-        print("spacing: {}".format(spacing))
-        print("volume.shape[0]: {}".format(volume.shape[0]))
+        #print("spacing: {}".format(spacing))
+        #print("volume.shape[0]: {}".format(volume.shape[0]))
 
         z_size = volume.shape[0]
         for z in range(0, z_size):
@@ -524,7 +551,7 @@ class NapariSTPT:
             slice_name = self.slice_names[z]
 
             spacing = (self.ds1[slice_name].attrs['scale'])
-            print(f"spacing {spacing}")
+            #print(f"spacing {spacing}")
             # fixed.SetSpacing([1.6*spacing[0],1.6*spacing[1]])
             fixed.SetSpacing([resolution*0.1*spacing[1],
                              resolution*0.1*spacing[0]])
@@ -832,6 +859,8 @@ class NapariSTPT:
             self.viewer.add_image([self.aligned_4], name='C4', scale=(
                 15, output_resolution, output_resolution), blending='additive', colormap='bop blue')
 
+        self.viewer.layers.remove('Shapes')
+        
     def CropToRegion(self):
 
         output_resolution = float(self.pixel_size.text())
@@ -881,6 +910,9 @@ class NapariSTPT:
             self.viewer.layers.remove('C4')
             self.viewer.add_image([self.aligned_4], name='C4', scale=(
                 15, output_resolution, output_resolution), blending='additive', colormap='bop blue', translate=(0, int(minX*output_resolution), int(minY*output_resolution)))
+            
+            
+        self.viewer.layers.remove('Shapes')
 
     def SelectFolder(self):
         file = str(QtWidgets.QFileDialog.getExistingDirectory())
@@ -906,15 +938,15 @@ class NapariSTPT:
             ) + str(self.comboBoxPath.currentText())+'/mos.zarr', consolidated=True)
             # print(ds.attrs["cube_reg"]['slice'])
             length = len(ds.attrs["multiscale"]['datasets'])
-            self.comboBoxResolution.clear()
-            # for n in reversed(range(0, length)):
-            for n in range(0, length):
-                print("adding resolution {}".format(
-                    ds.attrs["multiscale"]['datasets'][n]['level']))
-                self.comboBoxResolution.addItem(
-                    str(ds.attrs["multiscale"]['datasets'][n]['level']))
-                self.comboBoxResolution.setCurrentText(
-                    str(ds.attrs["multiscale"]['datasets'][n]['level']))
+            
+            #self.comboBoxResolution.clear()
+            #for n in range(0, length):
+            #    print("adding resolution {}".format(
+            #        ds.attrs["multiscale"]['datasets'][n]['level']))
+            #    self.comboBoxResolution.addItem(
+            #        str(ds.attrs["multiscale"]['datasets'][n]['level']))
+            #    self.comboBoxResolution.setCurrentText(
+            #        str(ds.attrs["multiscale"]['datasets'][n]['level']))
             try:
                 self.scroll.setValue(0)
                 self.image_slice.setText("0")
@@ -925,14 +957,14 @@ class NapariSTPT:
                 pass
         else:
             print("adding default resolution")
-            self.comboBoxResolution.clear()
-            self.comboBoxResolution.addItem(str(1))
-            self.comboBoxResolution.addItem(str(2))
-            self.comboBoxResolution.addItem(str(4))
-            self.comboBoxResolution.addItem(str(8))
-            self.comboBoxResolution.addItem(str(16))
-            self.comboBoxResolution.addItem(str(32))
-            self.comboBoxResolution.setCurrentText(str(32))
+            #self.comboBoxResolution.clear()
+            #self.comboBoxResolution.addItem(str(1))
+            #self.comboBoxResolution.addItem(str(2))
+            #self.comboBoxResolution.addItem(str(4))
+            #self.comboBoxResolution.addItem(str(8))
+            #self.comboBoxResolution.addItem(str(16))
+            #self.comboBoxResolution.addItem(str(32))
+            #self.comboBoxResolution.setCurrentText(str(32))
 
     def set_image_slice_value(self):
         value = self.scroll.value()
@@ -961,6 +993,14 @@ class NapariSTPT:
             if maxY < self.viewer.layers['Shapes'].data[0][i][1]:
                 maxY = self.viewer.layers['Shapes'].data[0][i][1]
 
+        minX2= minX / 2
+        #maxX = maxX / 2
+        minY2 = minY / 2
+        #maxY = maxY / 2
+                
+        maxX2 = (minX + (maxX - minX)/2)/2
+        maxY2 = (minY + (maxY - minY)/2)/2
+                
         print("{} {} {} {}".format(minX, maxX, minY, maxY))
         # return
         # prints a list of available models
@@ -990,6 +1030,9 @@ class NapariSTPT:
         # print(type(image))
         # return
         # normalize image
+        #ds1 = xr.open_zarr(self.search_folder.text(
+        #) + str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.1')
+        
         im12 = self.ds1[slice_name].sel(
             channel=2, type='mosaic', z=0).data * bscale + bzero
         im12 = np.array(im12[int(minX):int(maxX), int(minY):int(maxY)])
@@ -997,6 +1040,10 @@ class NapariSTPT:
             channel=3, type='mosaic', z=0).data * bscale + bzero
         im13 = np.array(im13[int(minX):int(maxX), int(minY):int(maxY)])
 
+        #im12 = np.array(self.aligned_2)
+        #im13 = np.array(self.aligned_3)
+        
+        
         # img = np.array(im1[(im1.shape[0]/2)-1000:(im1.shape[0]/2)+1000,(im1.shape[1]/2)-1000:(im1.shape[1]/2)+1000,])
 
         img = im12 - ((0.128 * im13) + 17.371)
@@ -1012,41 +1059,65 @@ class NapariSTPT:
         # return
         # predict segmentation
         labels, details = model.predict_instances(img, n_tiles=(10, 10))
-        # print(details['points'])
+        print(type(details['points'].shape[0]))
+        print(details['points'].shape)
+        #print(details)
+        #print(labels.shape)
+        #print(type(labels))
+        
+        max_value = np.max(labels)
+        print(f'max_value {max_value}')
+        
+        refined_labels = labels.copy()
+        minimum_cc_sum = 100000
+        for label in range(max_value+1):
+            if np.sum(refined_labels[labels == label]) < minimum_cc_sum:
+                refined_labels[labels == label] = 0
+                
+                
+        refined_labels2 = labels.copy()
+        minimum_cc_sum = 100000
+        for label in range(max_value+1):
+            if np.sum(refined_labels2[labels == label]) > minimum_cc_sum:
+                refined_labels2[labels == label] = 0
 
         # with napari.gui_qt():
         self.viewer.add_image(
-            img, name='img', blending='additive', translate=(int(minX), int(minY)))
+            img, name='img', blending='additive', scale=(1,1), translate=(int(minX), int(minY)))
 
         self.viewer.add_image(
-            im12, name='im12', blending='additive', translate=(int(minX), int(minY)))
+            im12, name='im12', blending='additive', scale=(1,1), translate=(int(minX), int(minY)))
         self.viewer.add_image(
-            im13, name='im13', blending='additive', translate=(int(minX), int(minY)))
+            im13, name='im13', blending='additive', scale=(1,1), translate=(int(minX), int(minY)))
         self.viewer.add_labels(
-            labels, name='labels', blending='additive', translate=(int(minX), int(minY)))
+            labels, name='labels', blending='additive', scale=(1,1), translate=(int(minX), int(minY)))
+        self.viewer.add_labels(
+            refined_labels, name='refined_labels', blending='additive', scale=(1,1), translate=(int(minX), int(minY)))
+        self.viewer.add_labels(
+            refined_labels2, name='refined_labels2', blending='additive', scale=(1,1), translate=(int(minX), int(minY)))
         self.viewer.add_points(
             details['points'], translate=(int(minX), int(minY)))
 
-    def SaveMovie(self):
+    #def SaveMovie(self):
 
-        widget = QtWidgets.QWidget()
-        fname = QtWidgets.QFileDialog.getSaveFileName(
-            widget, 'Save file as', 'c:\\', "Image files (*.gif *.mp4)")
+        #widget = QtWidgets.QWidget()
+        #fname = QtWidgets.QFileDialog.getSaveFileName(
+        #    widget, 'Save file as', 'c:\\', "Image files (*.gif *.mp4)")
 
-        if isinstance(fname, tuple):
-            print(fname[0])
-            filename, file_extension = os.path.splitext(fname[0])
-            if file_extension == '.mp4':
-                self.movie.make_movie(name='movie.mp4', resolution=300, fps=20)
-            if file_extension == '.gif':
-                self.movie.make_gif('S:/Tristan/gifmovie.gif')
+        #if isinstance(fname, tuple):
+        #    print(fname[0])
+        #    filename, file_extension = os.path.splitext(fname[0])
+        #    if file_extension == '.mp4':
+        #        self.movie.make_movie(name='movie.mp4', resolution=300, fps=20)
+        #    if file_extension == '.gif':
+        #        self.movie.make_gif('S:/Tristan/gifmovie.gif')
 
     def SaveVolume(self):
         widget = QtWidgets.QWidget()
-        fname = QtWidgets.QFileDialog.getSaveFileName(
+        fname, _ = QtWidgets.QFileDialog.getSaveFileName(
             widget, 'Save file as', 'c:\\', "Image files (*.tiff *.mha)")
 
-        if isinstance(fname, tuple):
+        if fname != "":
             print(fname[0])
 
             output_resolution = float(self.pixel_size.text())
@@ -1092,6 +1163,50 @@ class NapariSTPT:
             volume_itk = caster.Execute(volume_itk)
             sitk.WriteImage(volume_itk, fname[0])
 
+    def SaveSlice(self):
+        widget = QtWidgets.QWidget()
+        fname = QtWidgets.QFileDialog.getSaveFileName(
+            widget, 'Save file as', 'c:\\', "Image files (*.tiff *.mha)")
+
+        if isinstance(fname, tuple):
+            print(fname[0])
+
+            bscale = self.ds1.attrs['bscale']
+            bzero = self.ds1.attrs['bzero']
+
+            z = self.scroll.value()
+
+            self.slice_names = self.ds1.attrs['cube_reg']['slice']
+
+            self.scroll.setRange(0, len(self.slice_names)-1)
+            if z >= len(self.slice_names):
+                z = len(self.slice_names)-1
+                self.scroll.setValue(z)
+
+            slice_name = self.slice_names[z]
+
+
+
+            output_resolution = float(self.pixel_size.text())
+            spacing = (output_resolution, output_resolution)
+
+            #im1 = (self.ds1[slice_name].sel(
+            #    channel=3, type='mosaic', z=0).data * bscale + bzero).astype(dtype=np.float32)
+
+                
+            ds32 = xr.open_zarr(self.search_folder.text(
+                ) + str(self.comboBoxPath.currentText())+'/mos.zarr', group='l.32')
+            im32 = (ds32[slice_name].sel(
+                channel=3, type='mosaic', z=0).data * bscale + bzero).astype(dtype=np.float32)
+
+            image_itk = sitk.GetImageFromArray(im32)
+            image_itk.SetSpacing(spacing)
+            caster = sitk.CastImageFilter()
+            caster.SetOutputPixelType(sitk.sitkVectorFloat32)
+            image_itk = caster.Execute(image_itk)
+            sitk.WriteImage(image_itk, fname[0])
+
+
     def main(self):
 
         with napari.gui_qt():
@@ -1103,7 +1218,13 @@ class NapariSTPT:
 
             hbox = QtWidgets.QHBoxLayout()
             hbox.addWidget(QtWidgets.QLabel("Data folder:"))
-            self.search_folder = QtWidgets.QLineEdit('N:/stpt/')
+            
+            print(sys.platform)
+            if sys.platform == 'linux':
+                self.search_folder = QtWidgets.QLineEdit('/data/meds1_c/storage/processed/stpt')
+            else:
+                self.search_folder = QtWidgets.QLineEdit('N:/stpt/')
+            
             self.search_folder.setMaximumWidth(250)
             hbox.addWidget(self.search_folder)
             bSelectFolder = QtWidgets.QPushButton('Set folder')
@@ -1133,12 +1254,12 @@ class NapariSTPT:
             vbox.addLayout(hbox)
 
             hbox = QtWidgets.QHBoxLayout()
-            hbox.addWidget(QtWidgets.QLabel("Load level:"))
-            self.comboBoxResolution = QtWidgets.QComboBox()
-            self.comboBoxResolution.setMaximumWidth(100)
-            self.on_combobox_changed()
-            hbox.addWidget(self.comboBoxResolution)
-            hbox.addStretch(1)
+            #hbox.addWidget(QtWidgets.QLabel("Load level:"))
+            #self.comboBoxResolution = QtWidgets.QComboBox()
+            #self.comboBoxResolution.setMaximumWidth(100)
+            #self.on_combobox_changed()
+            #hbox.addWidget(self.comboBoxResolution)
+            #hbox.addStretch(1)
 
             hbox.addWidget(QtWidgets.QLabel("Output pixel size:"))
             self.pixel_size = QtWidgets.QLineEdit("7.5")
@@ -1249,18 +1370,24 @@ class NapariSTPT:
             vbox.addLayout(hbox)
 
 
+            hbox = QtWidgets.QHBoxLayout()
+
+            bSaveSlice = QtWidgets.QPushButton('Save slice')
+            bSaveSlice.setCheckable(True)
+            bSaveSlice.clicked.connect(self.SaveSlice)
+            hbox.addWidget(bSaveSlice)
+
             bSaveVolume = QtWidgets.QPushButton('Save volume')
             bSaveVolume.setCheckable(True)
             bSaveVolume.clicked.connect(self.SaveVolume)
-            hbox = QtWidgets.QHBoxLayout()
             hbox.addWidget(bSaveVolume)
 
-            bSaveMovie = QtWidgets.QPushButton('Save movie')
-            bSaveMovie.setCheckable(True)
-            bSaveMovie.clicked.connect(self.SaveMovie)
-            hbox.addWidget(bSaveMovie)
-            hbox.addStretch(1)
-            vbox.addLayout(hbox)
+            #bSaveMovie = QtWidgets.QPushButton('Save movie')
+            #bSaveMovie.setCheckable(True)
+            #bSaveMovie.clicked.connect(self.SaveMovie)
+            #hbox.addWidget(bSaveMovie)
+            #hbox.addStretch(1)
+            #vbox.addLayout(hbox)
 
             bStardist = QtWidgets.QPushButton('Run Stardist in shape')
             bStardist.setCheckable(True)
